@@ -7,6 +7,7 @@
 #include "gemmi/calculate.hpp"
 #include "gemmi/neighbor.hpp"
 #include "gemmi/select.hpp"  // count_atom_sites
+#include "gemmi/remarks.hpp"  // read_metadata_from_remarks
 #include <benchmark/benchmark.h>
 
 static const char* path;
@@ -18,6 +19,16 @@ static void read_pdb_file(benchmark::State& state) {
   }
 }
 
+static void read_pdb_remarks(benchmark::State& state) {
+  using namespace gemmi;
+  Structure st = read_pdb_file(path);
+  while (state.KeepRunning()) {
+    st.meta = gemmi::Metadata();
+    read_metadata_from_remarks(st);
+    benchmark::DoNotOptimize(st.meta);
+  }
+}
+
 static void find_atom_image(benchmark::State& state) {
   using namespace gemmi;
   Structure st = read_pdb_file(path);
@@ -51,7 +62,7 @@ static void neighbor_search_find(benchmark::State& state) {
   Position ref = model.chains.at(0).residues.at(2).atoms.at(0).pos;
   NeighborSearch ns(st.models.at(0), st.cell, 5.0);
   while (state.KeepRunning()) {
-    auto r = ns.find_atoms(ref, '\0', 4);
+    auto r = ns.find_atoms(ref, '\0', 0, 4);
     benchmark::DoNotOptimize(r);
   }
 }
@@ -65,7 +76,7 @@ static void neighbor_search_for_each(benchmark::State& state) {
   while (state.KeepRunning()) {
     double sum = 0;
     ns.for_each(ref, '\0', 4,
-                [&sum](NeighborSearch::Mark&, float d) { sum += d; });
+                [&sum](NeighborSearch::Mark&, double d) { sum += d; });
     benchmark::DoNotOptimize(sum);
   }
 }
@@ -150,6 +161,7 @@ int main(int argc, char** argv) {
            st.name.c_str(), count_atom_sites(st.models.at(0)));
   }
   benchmark::RegisterBenchmark("read_pdb_file", read_pdb_file);
+  benchmark::RegisterBenchmark("read_pdb_remarks", read_pdb_remarks);
   benchmark::RegisterBenchmark("find_atom_image", find_atom_image);
   benchmark::RegisterBenchmark("neighbor_search_ctor", neighbor_search_ctor);
   benchmark::RegisterBenchmark("neighbor_search_find", neighbor_search_find);
