@@ -15,62 +15,27 @@
 
 #include <algorithm>  // for swap
 #include <cctype>     // for isalpha
-#include <cstdio>     // for FILE, size_t
+#include <cstdio>     // for stdin, size_t
 #include <cstdlib>    // for strtol
-#include <cstring>    // for memcpy, strstr, strchr, strcmp
-#include <map>        // for map
-#include <string>     // for string
-#include <vector>     // for vector
+#include <cstring>    // for memcpy, strstr, strchr
 #include <unordered_map>
 
-#include "atox.hpp"     // for string_to_int
-#include "atof.hpp"     // for fast_from_chars
-#include "fail.hpp"     // for fail
 #include "fileutil.hpp" // for path_basename, file_open
 #include "input.hpp"    // for FileStream
-#include "model.hpp"
+#include "model.hpp"    // for Atom, Structure, ...
 #include "polyheur.hpp" // for assign_subchains
-#include "util.hpp"
+#include "remarks.hpp"  // for read_metadata_from_remarks, read_int, ...
 
 namespace gemmi {
 
 namespace pdb_impl {
 
-inline int read_int(const char* p, int field_length) {
-  return string_to_int(p, false, field_length);
-}
-
 template<int N> int read_base36(const char* p) {
   char zstr[N+1] = {0};
   std::memcpy(zstr, p, N);
   return std::strtol(zstr, nullptr, 36);
 }
 
-inline double read_double(const char* p, int field_length) {
-  double d = 0.;
-  // we don't check for errors here
-  fast_from_chars(p, p + field_length, d);
-  return d;
-}
-
-inline std::string read_string(const char* p, int field_length) {
-  // left trim
-  while (field_length != 0 && is_space(*p)) {
-    ++p;
-    --field_length;
-  }
-  // EOL/EOF ends the string
-  for (int i = 0; i < field_length; ++i)
-    if (p[i] == '\n' || p[i] == '\r' || p[i] == '\0') {
-      field_length = i;
-      break;
-    }
-  // right trim
-  while (field_length != 0 && is_space(p[field_length-1]))
-    --field_length;
-  return std::string(p, field_length);
-}
-
 // Compare the first 4 letters of s, ignoring case, with uppercase record.
 // Both args must have at least 3+1 chars. ' ' and NUL are equivalent in s.
 inline bool is_record_type(const char* s, const char* record) {
@@ -139,33 +104,6 @@ inline int read_serial(const char* ptr) {
                       : read_base36<5>(ptr) - 16796160 + 100000;
 }
 
-// "28-MAR-07" -> "2007-03-28"
-// (we also accept less standard format "28-Mar-2007" as used by BUSTER)
-// We do not check if the date is correct.
-// The returned value is one of:
-//   DDDD-DD-DD - possibly correct date,
-//   DDDD-xx-DD - unrecognized month,
-//   empty string - the digits were not there.
-inline std::string pdb_date_format_to_iso(const std::string& date) {
-  const char months[] = "JAN01FEB02MAR03APR04MAY05JUN06"
-                        "JUL07AUG08SEP09OCT10NOV11DEC122222";
-  if (date.size() < 9 || !is_digit(date[0]) || !is_digit(date[1]) ||
-                         !is_digit(date[7]) || !is_digit(date[8]))
-    return std::string();
-  std::string iso = "xxxx-xx-xx";
-  if (date.size() >= 11 && is_digit(date[9]) && is_digit(date[10])) {
-    std::memcpy(&iso[0], &date[7], 4);
-  } else {
-    std::memcpy(&iso[0], (date[7] > '6' ? "19" : "20"), 2);
-    std::memcpy(&iso[2], &date[7], 2);
-  }
-  char month[4] = {alpha_up(date[3]), alpha_up(date[4]), alpha_up(date[5]), '\0'};
-  if (const char* m = std::strstr(months, month))
-    std::memcpy(&iso[5], m + 3, 2);
-  std::memcpy(&iso[8], &date[0], 2);
-  return iso;
-}
-
 // move initials after comma, as in mmCIF (A.-B.DOE -> DOE, A.-B.), see
 // https://www.wwpdb.org/documentation/file-format-content/format33/sect2.html#AUTHOR
 inline void change_author_name_format_to_mmcif(std::string& name) {
@@ -261,24 +199,24 @@ void process_conn(Structure& st, const std::vector<std::string>& conn_records) {
       if (record.length() < 22)
         continue;
       const char* r = record.c_str();
-      std::string cname = read_string(r + 14, 2);
-      ResidueId rid = read_res_id(r + 17, r + 11);
-      for (Model& model : st.models)
-        if (Residue* res = model.find_residue(cname, rid))
-          res->is_cis = true;
+      CisPep cispep;
+      cispep.partner_c.chain_name = read_string(r + 14, 2);
+      cispep.partner_c.res_id = read_res_id(r + 17, r + 11);
+      cispep.partner_n.chain_name = read_string(r + 28, 2);
+      cispep.partner_n.res_id = read_res_id(r + 31, r + 25);
+      // In files with a single model in the PDB CISPEP modNum is 0,
+      // but _struct_mon_prot_cis.pdbx_PDB_model_num is 1.
+      cispep.model_str = st.models.size() == 1 ? st.models[0].name
+                                               : read_string(r + 43, 3);
+      cispep.reported_angle = read_double(r + 53, 6);
+      st.cispeps.push_back(cispep);
     }
   }
 }
 
-template<size_t N>
-inline bool same_str(const std::string& s, const char (&literal)[N]) {
-  return s.size() == N - 1 && std::strcmp(s.c_str(), literal) == 0;
-}
-
 template<typename Stream>
 Structure read_pdb_from_stream(Stream&& stream, const std::string& source,
                                const PdbReadOptions& options) {
-  using namespace pdb_impl;
   int line_num = 0;
   auto wrong = [&line_num](const std::string& msg) {
     fail("Problem in line " + std::to_string(line_num) + ": " + msg);
@@ -365,6 +303,10 @@ Structure read_pdb_from_stream(Stream&& stream, const std::string& source,
       // never have 4-character names, so H is assumed.
       else if (alpha_up(line[12]) == 'H' && line[15] != ' ')
         atom.element = El::H;
+      // Similarly Deuterium (DXXX), but here alternatives are Dy, Db and Ds.
+      // Only Dysprosium is present in the PDB - in a single entry as of 2022.
+      else if (alpha_up(line[12]) == 'D' && line[15] != ' ')
+        atom.element = El::D;
       // Old versions of the PDB format had hydrogen names such as "1HB ".
       // Some MD files use similar names for other elements ("1C4A" -> C).
       else if (is_digit(line[12]))
@@ -402,7 +344,7 @@ Structure read_pdb_from_stream(Stream&& stream, const std::string& source,
         continue;
       int num = read_int(line + 7, 3);
       // By default, we only look for resolution and REMARK 350.
-      // Other parsing of remarks is in interpret_remarks().
+      // Other remarks are parsed in read_metadata_from_remarks()
       if (num == 2) {
         if (st.resolution == 0.0 && std::strstr(line, "ANGSTROM"))
           st.resolution = read_double(line + 23, 7);
@@ -469,6 +411,27 @@ Structure read_pdb_from_stream(Stream&& stream, const std::string& source,
           ent.full_sequence.emplace_back(res_name);
       }
 
+    } else if (is_record_type(line, "MODRES")) {
+      ModRes modres;
+      modres.chain_name = read_string(line + 15, 2);
+      modres.res_id = read_res_id(line + 18, line + 12);
+      modres.parent_comp_id = read_string(line + 24, 3);
+      if (len >= 30)
+        // this field is named comment in PDB spec, but details in mmCIF
+        modres.details = read_string(line + 29, 41);
+      // Refmac's extension: 73-80 mod_id
+      // Check for spaces to make sure it's not an overflowed comment
+      if (len >= 73 && line[70] == ' ' && line[71] == ' ')
+        modres.mod_id = read_string(line + 72, 6);
+      st.mod_residues.push_back(modres);
+
+    } else if (is_record_type(line, "HETNAM")) {
+      if (len > 71 && line[70] == ' ') {
+        std::string full_code = read_string(line + 71, 8);
+        if (!full_code.empty())
+          st.shortened_ccd_codes.push_back({full_code, read_string(line + 11, 3)});
+      }
+
     } else if (is_record_type(line, "DBREF")) { // DBREF or DBREF1 or DBREF2
       std::string chain_name = read_string(line+11, 2);
       Entity& ent = impl::find_or_add(st.entities, chain_name);
@@ -658,10 +621,9 @@ Structure read_pdb_from_stream(Stream&& stream, const std::string& source,
   if (st.models.empty())
     st.models.emplace_back("1");
 
-  for (Model& mod : st.models)
-    for (Chain& ch : mod.chains)
-      if (ch.residues[0].entity_type != EntityType::Unknown)
-        assign_subchain_names(ch);
+  // Here we assign Residue::subchain, but for chains with all
+  // Residue::entity_type assigned, i.e. for chains with TER.
+  assign_subchains(st, /*force=*/false, /*fail_if_unknown=*/false);
 
   for (Chain& ch : st.models[0].chains)
     if (Entity* entity = st.get_entity(ch.name))
@@ -675,6 +637,11 @@ Structure read_pdb_from_stream(Stream&& stream, const std::string& source,
   for (std::string& name : st.meta.authors)
     change_author_name_format_to_mmcif(name);
 
+  if (!options.skip_remarks)
+    read_metadata_from_remarks(st);
+
+  restore_full_ccd_codes(st);
+
   return st;
 }
 
