@@ -177,6 +177,9 @@ struct Op {
   static constexpr Op identity() {
     return {{DEN,0,0, 0,DEN,0, 0,0,DEN}, {0,0,0}};
   }
+  static constexpr Op::Rot inversion_rot() {
+    return {-DEN,0,0, 0,-DEN,0, 0,0,-DEN};
+  }
   bool operator<(const Op& rhs) const {
     return std::tie(rot, tran) < std::tie(rhs.rot, rhs.tran);
   }
@@ -482,7 +485,7 @@ struct GroupOps {
   }
 
   bool is_centrosymmetric() const {
-    return find_by_rotation({-Op::DEN,0,0, 0,-Op::DEN,0, 0,0,-Op::DEN}) != nullptr;
+    return find_by_rotation(Op::inversion_rot()) != nullptr;
   }
 
   bool is_reflection_centric(const Op::Miller& hkl) const {
@@ -1033,6 +1036,24 @@ inline bool is_symmorphic(int space_group_number) {
   return (point_group_index_and_category(space_group_number) & 0x80) != 0;
 }
 
+/// Inversion center of the Euclidean normalizer that is not at the origin of
+/// reference settings. Returns (0,0,0) if absent. Based on tables in ch. 3.5
+/// of ITA (2016) doi:10.1107/97809553602060000933 (column "Inversion through
+/// a centre at").
+inline Op::Tran nonzero_inversion_center(int space_group_number) {
+  constexpr int D = Op::DEN;
+  switch (space_group_number) {
+    case 43: return {D/8, D/8, 0};
+    case 80: return {D/4, 0, 0};
+    case 98: return {D/4, 0, D/8};
+    case 109: return {D/4, 0, 0};
+    case 110: return {D/4, 0, 0};
+    case 122: return {D/4, 0, D/8};
+    case 210: return {D/8, D/8, D/8};
+    default: return {0, 0, 0};
+  }
+}
+
 // Generated by tools/gen_sg_table.py.
 inline const char* get_basisop(int basisop_idx) {
   static const char* basisops[49] = {
@@ -1182,6 +1203,19 @@ struct SpaceGroup { // typically 44 bytes
     return {gemmi::centred_to_primitive(centring_type()), {0,0,0}};
   }
 
+  /// Returns change-of-hand operator. Compatible with similar sgtbx function.
+  Op change_of_hand_op() const {
+    if (is_centrosymmetric())
+      return Op::identity();
+    Op::Tran t = nonzero_inversion_center(number);
+    Op op{Op::inversion_rot(), {2*t[0], 2*t[1], 2*t[2]}};
+    if (!is_reference_setting()) {
+      Op b = basisop();
+      op = b.combine(op).combine(b.inverse());
+    }
+    return op;
+  }
+
   GroupOps operations() const { return symops_from_hall(hall); }
 };
 
@@ -1531,15 +1565,15 @@ const SpaceGroup Tables_<Dummy>::main[559] = {
   { 67,    0, "B m a m"   ,   0, "a-cb", "-B 2a 2"       , 5 }, // 320
   { 68,   68, "C c c a"   , '1',     "", "C 2 2 -1ac"    , 24}, // 321
   { 68,    0, "C c c a"   , '2',     "", "-C 2a 2ac"     , 0 }, // 322
-  { 68,    0, "C c c b"   , '1', "ba-c", "C 2 2 -1ac"    , 24}, // 323
+  { 68,    0, "C c c b"   , '1', "ba-c", "C 2 2 -1ac"    , 24}, // 323 (==321)
   { 68,    0, "C c c b"   , '2', "ba-c", "-C 2a 2c"      , 21}, // 324
   { 68,    0, "A b a a"   , '1',  "cab", "A 2 2 -1ab"    , 25}, // 325
   { 68,    0, "A b a a"   , '2',  "cab", "-A 2a 2b"      , 1 }, // 326
-  { 68,    0, "A c a a"   , '1', "-cba", "A 2 2 -1ab"    , 25}, // 327
+  { 68,    0, "A c a a"   , '1', "-cba", "A 2 2 -1ab"    , 25}, // 327 (==325)
   { 68,    0, "A c a a"   , '2', "-cba", "-A 2ab 2b"     , 3 }, // 328
   { 68,    0, "B b c b"   , '1',  "bca", "B 2 2 -1ab"    , 26}, // 329
   { 68,    0, "B b c b"   , '2',  "bca", "-B 2ab 2b"     , 2 }, // 330
-  { 68,    0, "B b a b"   , '1', "a-cb", "B 2 2 -1ab"    , 26}, // 331
+  { 68,    0, "B b a b"   , '1', "a-cb", "B 2 2 -1ab"    , 26}, // 331 (==329)
   { 68,    0, "B b a b"   , '2', "a-cb", "-B 2b 2ab"     , 5 }, // 332
   { 69,   69, "F m m m"   ,   0,     "", "-F 2 2"        , 0 }, // 333
   { 70,   70, "F d d d"   , '1',     "", "F 2 2 -1d"     , 27}, // 334
@@ -1768,7 +1802,8 @@ const SpaceGroup Tables_<Dummy>::main[559] = {
   {  3,    0, "C 1 1 2"  ,    0,     "", "C 2"           , 46}, // 549
   {  4,    0, "C 1 1 21"  ,   0,     "", "C 2c"          , 46}, // 550
   { 12,    0, "F 1 2/m 1" ,   0,     "", "-F 2y"         , 47}, // 551
-  { 64,    0, "A b a m"   ,   0,     "", "-A 2 2ab"      , 3 }, // 552
+  // orthorhombic
+  { 64,    0, "A b a m"   ,   0,     "", "-A 2 2ab"      , 3 }, // 552 (==306)
   // tetragonal - enlarged C- and F-centred unit cells
   { 89,    0, "C 4 2 2" ,     0,     "", "C 4 2"         , 48}, // 553
   { 90,    0, "C 4 2 21" ,    0,     "", "C 4a 2"        , 48}, // 554
@@ -1966,7 +2001,7 @@ inline const SpaceGroup* find_spacegroup_by_ops(const GroupOps& gops) {
 // The same 12 choices of ASU as in CCP4 symlib and cctbx.
 struct ReciprocalAsu {
   int idx;
-  Op::Rot rot;
+  Op::Rot rot{};  // value-initialized only to avoid -Wmaybe-uninitialized
   bool is_ref;
 
   ReciprocalAsu(const SpaceGroup* sg, bool tnt=false) {
@@ -2042,8 +2077,8 @@ struct ReciprocalAsu {
     unreachable();
   }
 
-  // Returns hkl in asu and MTZ ISYM - 2*n-1 for reflections in the positive
-  // asu (I+ of a Friedel pair), 2*n for reflections in the negative asu (I-).
+  /// Returns hkl in asu and MTZ ISYM - 2*n-1 for reflections in the positive
+  /// asu (I+ of a Friedel pair), 2*n for reflections in the negative asu (I-).
   std::pair<Op::Miller,int> to_asu(const Op::Miller& hkl, const GroupOps& gops) const {
     int isym = 0;
     for (const Op& op : gops.sym_ops) {
@@ -2058,6 +2093,22 @@ struct ReciprocalAsu {
     }
     fail("Oops, maybe inconsistent GroupOps?");
   }
+  /// Similar to to_asu(), but the second returned value is sign: true for + or centric
+  std::pair<Op::Miller,bool> to_asu_sign(const Op::Miller& hkl, const GroupOps& gops) const {
+    std::pair<Op::Miller,bool> neg = {{0,0,0}, true};
+    for (const Op& op : gops.sym_ops) {
+      Op::Miller new_hkl = op.apply_to_hkl_without_division(hkl);
+      if (is_in(new_hkl))
+        return {Op::divide_hkl_by_DEN(new_hkl), true};
+      Op::Miller negated_new_hkl{{-new_hkl[0], -new_hkl[1], -new_hkl[2]}};
+      if (is_in(negated_new_hkl))
+        // don't return it yet, because for centric reflection we prefer (+)
+        neg = {Op::divide_hkl_by_DEN(negated_new_hkl), false};
+    }
+    if (neg.second)
+      fail("Oops, maybe inconsistent GroupOps?");
+    return neg;
+  }
 };
 
 } // namespace gemmi
