@@ -5,6 +5,7 @@
 #include "gemmi/modify.hpp"     // for remove_alternative_conformations
 #include "gemmi/polyheur.hpp"   // for one_letter_code, trim_to_alanine
 #include "gemmi/assembly.hpp"   // for expand_ncs, HowToNameCopiedChain
+#include "gemmi/select.hpp"     // for Selection
 #include "tostr.hpp"
 
 #include "common.h"
@@ -59,6 +60,7 @@ void add_mol(py::module& m) {
   py::class_<Model> pyModel(m, "Model");
   py::class_<ResidueSpan> pyResidueSpan(m, "ResidueSpan");
   py::class_<ResidueGroup, ResidueSpan> pyResidueGroup(m, "ResidueGroup");
+  py::class_<Selection> pySelection(m, "Selection");
 
   py::enum_<HowToNameCopiedChain>(m, "HowToNameCopiedChain")
     .value("Short", HowToNameCopiedChain::Short)
@@ -82,6 +84,22 @@ void add_mol(py::module& m) {
 
   py::bind_map<info_map_type>(m, "InfoMap");
 
+  py::class_<CRA>(m, "CRA")
+    .def_readonly("chain", &CRA::chain)
+    .def_readonly("residue", &CRA::residue)
+    .def_readonly("atom", &CRA::atom)
+    .def("atom_matches", [](const CRA& self, const AtomAddress& addr) {
+        return atom_matches(self, addr);
+    })
+    .def("__str__", [](const CRA& self) { return atom_str(self); })
+    .def("__repr__", [](const CRA& self) {
+        return tostr("<gemmi.CRA ", atom_str(self), '>');
+    });
+
+  py::class_<CraProxy>(m, "CraGenerator")
+    .def("__iter__", [](CraProxy& self) { return py::make_iterator(self); },
+         py::keep_alive<0, 1>());
+
   py::class_<Structure> structure(m, "Structure");
   structure
     .def(py::init<>())
@@ -93,10 +111,12 @@ void add_mol(py::module& m) {
     .def_readwrite("input_format", &Structure::input_format)
     .def_readwrite("entities", &Structure::entities)
     .def_readwrite("connections", &Structure::connections)
+    .def_readwrite("cispeps", &Structure::cispeps)
     .def_readwrite("helices", &Structure::helices)
     .def_readwrite("sheets", &Structure::sheets)
     .def_readwrite("assemblies", &Structure::assemblies)
     .def_readwrite("meta", &Structure::meta)
+    .def_readwrite("has_d_fraction", &Structure::has_d_fraction)
     .def_readwrite("info", &Structure::info)
     .def_readwrite("raw_remarks", &Structure::raw_remarks)
     .def("find_spacegroup", &Structure::find_spacegroup)
@@ -119,6 +139,9 @@ void add_mol(py::module& m) {
     .def("__delitem__", &remove_children<Structure>)
     .def("__delitem__", &Structure::remove_model, py::arg("name"))
     .def("__setitem__", &set_child<Structure, Model>)
+    .def("find_connection_by_cra", [](Structure& st, CRA cra1, CRA cra2) {
+        return st.find_connection_by_cra(cra1, cra2);
+    }, py::return_value_policy::reference_internal)
     .def("find_connection", &Structure::find_connection,
          py::arg("partner1"), py::arg("partner2"),
          py::return_value_policy::reference_internal)
@@ -134,8 +157,8 @@ void add_mol(py::module& m) {
     .def("setup_cell_images", &Structure::setup_cell_images)
     .def("add_entity_types", (void (*)(Structure&, bool)) &add_entity_types,
          py::arg("overwrite")=false)
-    .def("assign_subchains", (void (*)(Structure&, bool)) &assign_subchains,
-         py::arg("overwrite")=false)
+    .def("assign_subchains", &assign_subchains,
+         py::arg("force")=false, py::arg("fail_if_unknown")=true)
     .def("ensure_entities", &ensure_entities)
     .def("deduplicate_entities", &deduplicate_entities)
     .def("setup_entities", &setup_entities)
@@ -144,6 +167,8 @@ void add_mol(py::module& m) {
     .def("remove_hydrogens", remove_hydrogens<Structure>)
     .def("remove_waters", remove_waters<Structure>)
     .def("remove_ligands_and_waters", remove_ligands_and_waters<Structure>)
+    .def("store_deuterium_as_fraction", &store_deuterium_as_fraction)
+    .def("assign_serial_numbers", (void (*)(Structure&)) &assign_serial_numbers)
     .def("shorten_chain_names", &shorten_chain_names)
     .def("expand_ncs", &expand_ncs, py::arg("how"))
     .def("transform_to_assembly",
@@ -160,22 +185,6 @@ void add_mol(py::module& m) {
     add_assign_label_seq_id(structure);
     add_write(m, structure);
 
-  py::class_<CRA>(m, "CRA")
-    .def_readonly("chain", &CRA::chain)
-    .def_readonly("residue", &CRA::residue)
-    .def_readonly("atom", &CRA::atom)
-    .def("atom_matches", [](const CRA& self, const AtomAddress& addr) {
-        return atom_matches(self, addr);
-    })
-    .def("__str__", [](const CRA& self) { return atom_str(self); })
-    .def("__repr__", [](const CRA& self) {
-        return tostr("<gemmi.CRA ", atom_str(self), '>');
-    });
-
-  py::class_<CraProxy>(m, "CraGenerator")
-    .def("__iter__", [](CraProxy& self) { return py::make_iterator(self); },
-         py::keep_alive<0, 1>());
-
   pyModel
     .def(py::init<std::string>())
     .def_readwrite("name", &Model::name)
@@ -229,16 +238,16 @@ void add_mol(py::module& m) {
     .def("remove_hydrogens", remove_hydrogens<Model>)
     .def("remove_waters", remove_waters<Model>)
     .def("remove_ligands_and_waters", remove_ligands_and_waters<Model>)
-    .def("count_atom_sites", &count_atom_sites<Model>)
+    .def("has_hydrogen", &has_hydrogen<Model>)
+    .def("count_atom_sites", &count_atom_sites<Model>, py::arg("sel")=nullptr)
+    // deprecated, use has_hydrogen() or count_atom_sites(Selection('[H,D]'))
     .def("count_hydrogen_sites", &count_hydrogen_sites<Model>)
-    .def("count_occupancies", &count_occupancies<Model>)
+    .def("count_occupancies", &count_occupancies<Model>, py::arg("sel")=nullptr)
     .def("calculate_mass", &calculate_mass<Model>)
     .def("calculate_center_of_mass", [](const Model& self) {
         return calculate_center_of_mass(self).get();
     })
     .def("transform_pos_and_adp", transform_pos_and_adp<Model>, py::arg("tr"))
-    // the same function under differnt name - for compability
-    .def("transform", transform_pos_and_adp<Model>, py::arg("tr"))
     .def("split_chains_by_segments", &split_chains_by_segments)
     .def("clone", [](const Model& self) { return new Model(self); })
     .def("__repr__", [](const Model& self) {
@@ -297,15 +306,14 @@ void add_mol(py::module& m) {
          py::keep_alive<0, 1>())
     .def("get_subchain", (ResidueSpan (Chain::*)(const std::string&)) &Chain::get_subchain,
          py::keep_alive<0, 1>())
-    .def("has_subchains_assigned", &has_subchains_assigned)
     .def("previous_residue", &Chain::previous_residue,
          py::return_value_policy::reference_internal)
     .def("next_residue", &Chain::next_residue,
          py::return_value_policy::reference_internal)
     .def("append_residues", &Chain::append_residues,
          py::arg("new_residues"), py::arg("min_sep")=0)
-    .def("count_atom_sites", &count_atom_sites<Chain>)
-    .def("count_occupancies", &count_occupancies<Chain>)
+    .def("count_atom_sites", &count_atom_sites<Chain>, py::arg("sel")=nullptr)
+    .def("count_occupancies", &count_occupancies<Chain>, py::arg("sel")=nullptr)
     .def("calculate_mass", &calculate_mass<Chain>)
     .def("calculate_center_of_mass", [](const Chain& self) {
           return calculate_center_of_mass(self).get();
@@ -406,10 +414,15 @@ void add_mol(py::module& m) {
     .def(py::init<>())
     .def_readwrite("subchain", &Residue::subchain)
     .def_readonly("entity_id", &Residue::entity_id)
+    .def_readwrite("label_seq", &Residue::label_seq)
     .def_readwrite("entity_type", &Residue::entity_type)
     .def_readwrite("het_flag", &Residue::het_flag)
     .def_readwrite("flag", &Residue::flag)
-    .def_readwrite("label_seq", &Residue::label_seq)
+    .def_property_readonly("sifts_unp", [](const Residue& self) {
+        return py::make_tuple(self.sifts_unp.res,
+                              self.sifts_unp.acc_index,
+                              self.sifts_unp.num);
+    })
     .def("__len__", [](const Residue& res) { return res.atoms.size(); })
     .def("__contains__", [](const Residue& res, const std::string& name) {
         return res.find_atom(name, '*') != nullptr;
@@ -451,6 +464,7 @@ void add_mol(py::module& m) {
 
   py::enum_<CalcFlag>(m, "CalcFlag")
     .value("NotSet", CalcFlag::NotSet)
+    .value("NoHydrogen", CalcFlag::NoHydrogen)
     .value("Determined", CalcFlag::Determined)
     .value("Calculated", CalcFlag::Calculated)
     .value("Dummy", CalcFlag::Dummy);
@@ -466,6 +480,7 @@ void add_mol(py::module& m) {
     .def_readwrite("b_iso", &Atom::b_iso)
     .def_readwrite("aniso", &Atom::aniso)
     .def_readwrite("serial", &Atom::serial)
+    .def_readwrite("fraction", &Atom::fraction)
     .def_readwrite("calc_flag", &Atom::calc_flag)
     .def_readwrite("flag", &Atom::flag)
     .def_readwrite("tls_group_id", &Atom::tls_group_id)
@@ -506,4 +521,56 @@ void add_mol(py::module& m) {
                             HowToNameCopiedChain how) {
         return make_assembly(assembly, model, how, nullptr);
   });
+  m.def("merge_atoms_in_expanded_model", &merge_atoms_in_expanded_model,
+        py::arg("model"), py::arg("cell"), py::arg("max_dist")=0.2);
+
+  // select.hpp
+  py::class_<FilterProxy<Selection, Model>> pySelectionModelsProxy(m, "SelectionModelsProxy");
+  py::class_<FilterProxy<Selection, Chain>> pySelectionChainsProxy(m, "SelectionChainsProxy");
+  py::class_<FilterProxy<Selection, Residue>> pySelectionResiduesProxy(m, "SelectionResiduesProxy");
+  py::class_<FilterProxy<Selection, Atom>> pySelectionAtomsProxy(m, "SelectionAtomsProxy");
+
+  pySelection
+    .def(py::init<>())
+    .def(py::init<const std::string&>())
+    .def("models", &Selection::models)
+    .def("chains", &Selection::chains)
+    .def("residues", &Selection::residues)
+    .def("atoms", &Selection::atoms)
+    .def("first_in_model", &Selection::first_in_model,
+         py::keep_alive<1, 2>())
+    .def("first", &Selection::first, py::return_value_policy::reference,
+         py::keep_alive<1, 2>())
+    .def("str", &Selection::str)
+    .def("set_residue_flags", &Selection::set_residue_flags)
+    .def("set_atom_flags", &Selection::set_atom_flags)
+    .def("copy_model_selection", &Selection::copy_selection<Model>)
+    .def("copy_structure_selection", &Selection::copy_selection<Structure>)
+    .def("remove_selected", &Selection::remove_selected<Structure>)
+    .def("remove_selected", &Selection::remove_selected<Model>)
+    .def("remove_not_selected", &Selection::remove_not_selected<Structure>)
+    .def("remove_not_selected", &Selection::remove_not_selected<Model>)
+    .def("__repr__", [](const Selection& self) {
+        return "<gemmi.Selection CID: " + self.str() + ">";
+    });
+
+  pySelectionModelsProxy
+    .def("__iter__", [](FilterProxy<Selection, Model>& self) {
+        return py::make_iterator(self);
+    }, py::keep_alive<0, 1>());
+
+  pySelectionChainsProxy
+    .def("__iter__", [](FilterProxy<Selection, Chain>& self) {
+        return py::make_iterator(self);
+    }, py::keep_alive<0, 1>());
+
+  pySelectionResiduesProxy
+    .def("__iter__", [](FilterProxy<Selection, Residue>& self) {
+        return py::make_iterator(self);
+    }, py::keep_alive<0, 1>());
+
+  pySelectionAtomsProxy
+    .def("__iter__", [](FilterProxy<Selection, Atom>& self) {
+        return py::make_iterator(self);
+    }, py::keep_alive<0, 1>());
 }
