@@ -1,7 +1,6 @@
 // Copyright 2019 Global Phasing Ltd.
 
 #include "gemmi/mtz.hpp"
-#include "gemmi/reindex.hpp"  // for reindex_mtz
 #include "gemmi/fourier.hpp"
 #include "gemmi/gz.hpp"
 #include "tostr.hpp"
@@ -17,7 +16,6 @@ using namespace gemmi;
 PYBIND11_MAKE_OPAQUE(std::vector<Mtz::Dataset>)
 PYBIND11_MAKE_OPAQUE(std::vector<Mtz::Column>)
 PYBIND11_MAKE_OPAQUE(std::vector<Mtz::Batch>)
-PYBIND11_MAKE_OPAQUE(std::vector<const Mtz::Column*>)
 
 namespace gemmi {
   // operator<< is used by stl_bind for vector's __repr__
@@ -26,11 +24,6 @@ namespace gemmi {
        << '/' << ds.crystal_name << '/' << ds.dataset_name << '>';
     return os;
   }
-
-  inline std::ostream& operator<< (std::ostream& os, const Mtz::Column* col) {
-    os << "<gemmi.Mtz.Column " << col->label << " type " << col->type << '>';
-    return os;
-  }
 }
 
 template<typename F>
@@ -73,7 +66,6 @@ void add_mtz(py::module& m) {
   py::bind_vector<std::vector<Mtz::Dataset>>(m, "MtzDatasets");
   py::bind_vector<std::vector<Mtz::Column>>(m, "MtzColumns");
   py::bind_vector<std::vector<Mtz::Batch>>(m, "MtzBatches");
-  py::bind_vector<std::vector<const Mtz::Column*>>(m, "MtzColumnRefs");
 
   mtz
     .def(py::init<bool>(), py::arg("with_base")=false)
@@ -119,7 +111,7 @@ void add_mtz(py::module& m) {
     .def("rfree_column", (Mtz::Column* (Mtz::*)()) &Mtz::rfree_column,
           py::return_value_policy::reference_internal)
     .def("columns_with_type", &Mtz::columns_with_type,
-         py::arg("type"), py::keep_alive<0, 1>())
+         py::arg("type"), py::return_value_policy::reference_internal)
     .def("column_labels", [](const Mtz& self) {
         std::vector<std::string> labels;
         labels.reserve(self.columns.size());
@@ -251,14 +243,14 @@ void add_mtz(py::module& m) {
              self.data[row*ncol+col] = r(row, col);
     }, py::arg("array"))
     .def("update_reso", &Mtz::update_reso)
-    .def("sort", &Mtz::sort)
-    .def("ensure_asu", &Mtz::ensure_asu)
+    .def("sort", &Mtz::sort, py::arg("use_first")=3)
+    .def("ensure_asu", &Mtz::ensure_asu, py::arg("tnt_asu")=false)
     .def("switch_to_original_hkl", &Mtz::switch_to_original_hkl)
     .def("switch_to_asu_hkl", &Mtz::switch_to_asu_hkl)
     .def("write_to_file", &Mtz::write_to_file, py::arg("path"))
     .def("reindex", [](Mtz& self, const Op& op) {
         std::ostringstream out;
-        reindex_mtz(self, op, &out);
+        self.reindex(op, &out);
         return out.str();
     }, py::arg("op"))
     .def("__repr__", [](const Mtz& self) {
@@ -303,9 +295,12 @@ void add_mtz(py::module& m) {
     .def("__iter__", [](Mtz::Column& self) {
         return py::make_iterator(self);
     }, py::keep_alive<0, 1>())
-    .def("__repr__", [](const Mtz::Column& self) { return tostr(&self); })
+    .def("__repr__", [](const Mtz::Column& self) {
+        return cat("<gemmi.Mtz.Column ", self.label, " type ", self.type, '>');
+    })
     ;
   pyMtzBatch
+    .def(py::init<>())
     .def_readwrite("number", &Mtz::Batch::number)
     .def_property_readonly("dataset_id", &Mtz::Batch::dataset_id)
     .def_readwrite("title", &Mtz::Batch::title)
